// MakeTerrainDoc.cpp : implementation of the CMakeTerrainDoc class
//

#include "stdafx.h"
#include "MakeTerrain.h"

#include "MakeTerrainDoc.h"
#include "math.h"
#include "DlgConfigTerrain.h"


#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

char ligne [1024];
char chaine [1024];
int TabFace [5];
extern BOOL bRender; 
/////////////////////////////////////////////////////////////////////////////
// CMakeTerrainDoc
extern CMakeTerrainDoc * pGlobalDoc;
extern GLfloat PIMULMISSILAUTO;
extern int TypePlaquage;
extern int NCHUNK;



// Liste des varaibles gloables

IMPLEMENT_DYNCREATE(CMakeTerrainDoc, CDocument)

BEGIN_MESSAGE_MAP(CMakeTerrainDoc, CDocument)
	//{{AFX_MSG_MAP(CMakeTerrainDoc)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMakeTerrainDoc construction/destruction


CMakeTerrainDoc::CMakeTerrainDoc()
{

	// Au dpart le tableau est NULL
	Tab = NULL;
	Face = NULL;
	pTexture = NULL;


	// Les autres attributs
	NbPoint = NPOINT;
	Largeur = 20;

	NbCase = (NbPoint-1)*(NbPoint-1);
	NbFacette = NbCase * 2;


	Hauteur = 255;

	ip = 0;
	jp = 0;

	imousep = 0;
	jmousep = 0;

	pGlobalDoc = this;



}

CMakeTerrainDoc::~CMakeTerrainDoc()
{
	// Detruire la mmoire
	FreeMemory ();
}

// Destruction de la mmoire alloue
void CMakeTerrainDoc::FreeMemory()
{
	int i;
	
	if (Tab != NULL)
	{
		for (i = 0; i < NbPoint; i++)  if (Tab [i] != NULL) free (Tab [i]);
		free (Tab);
	}
	if (Face != NULL) free (Face);
}

BOOL CMakeTerrainDoc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;


	// Cration d'un nouveau terrain
	return NewTerrain (NbPoint);
}

// Cration d'un nouveau terrain
BOOL CMakeTerrainDoc :: NewTerrain (int NbPointInit)
{
	POINTASC_TERRE * pTTer;
	int i;


	// Detruire la mmoire avant de crer un nouveau terrain
	FreeMemory ();

	// Initialisation du nombre de point
	NbPoint = NbPointInit;
	
	if (NCHUNK > NbPoint)NCHUNK = NbPoint / 4;
	NbCase = (NbPoint-1)*(NbPoint-1);
	NbFacette = NbCase * 2;
	

	Tab = (POINTASC_TERRE **) malloc (sizeof (POINTASC_TERRE *) * NbPoint);

	if (Tab == NULL) return FALSE;
	memset (Tab,0,sizeof (POINTASC_TERRE *) * NbPoint);

	for (i = 0; i < NbPoint; i++)
	{
		pTTer = (POINTASC_TERRE *) malloc (sizeof (POINTASC_TERRE) * NbPoint);
		if (pTTer == NULL) return FALSE;
		Tab [i] = pTTer;
	}

	Face = (FACETTE_TERRE *) malloc (sizeof (FACETTE_TERRE) * NbFacette);
	if (Face == NULL) return FALSE;

	// Crer la texture pour la vue 2D
	if (pTexture != NULL) delete pTexture;
	pTexture = new TextureInfo (NbPoint,NbPoint);
	pTexture->CreateBitmap (BmpLand);

	// Initialisation du terrain
	InitFacette ();
	Reset ();



	return TRUE;
}


POINTASC CMakeTerrainDoc :: Normal (float x,float y,float z,float vecx,float vecy,float vecz)
{
	POINTASC pt;
	pt.x = y * vecz - vecy * z;
	pt.y = z * vecx - vecz * x;
	pt.z = x * vecy - vecx * y;
	return pt;
}





/////////////////////////////////////////////////////////////////////////////
// CMakeTerrainDoc serialization

void CMakeTerrainDoc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CMakeTerrainDoc diagnostics

#ifdef _DEBUG
void CMakeTerrainDoc::AssertValid() const
{
	CDocument::AssertValid();
}

void CMakeTerrainDoc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMakeTerrainDoc commands

// Cette fonction permet d'initialiser le terrain plat
void CMakeTerrainDoc::Reset (BOOL bResetAll)
{
	int i,j;
	
	// Position de depart en x et z
	Dep = -NbPoint * Largeur / 2;


	for (i = 0; i < NbPoint; i ++)
	{
		for (j = 0; j < NbPoint; j ++)
		{
			Tab [i] [j].x = (GLfloat) (Dep + (i * Largeur));
			Tab [i] [j].z =(GLfloat) (Dep + (j * Largeur));
			if (bResetAll) Tab [i] [j].y = 0;
			
			SetPointColor (i,j,Tab [i] [j].y);

			Tab [i] [j].tx = (float)i/NbPoint;
			Tab [i] [j].ty = (float)j/NbPoint;

		}
	}

	// Recalculer les normales de facettes
	NormalFacette ();

	for (i = 0; i < NbPoint; i ++)
		for (j = 0; j < NbPoint; j ++)
			NormalPoint (i,j);

}



// Initialise le tableau de FACETTE_TERRE
void CMakeTerrainDoc :: InitFacette ()
{
	int i;
	int j;
	int no;
	int Case;

	// 16384
	no = 0;

	for (j = 0; j < NbPoint-1; j++)
	{
		for (i = 0; i < NbPoint-1; i++)
		{
			// Deuxieme triangle
			Face [no].a.i = i+1;
			Face [no].a.j = j+1;

			Face [no].b.i = i;
			Face [no].b.j = j;

			Face [no].c.i = i;
			Face [no].c.j = j+1;

			Face [no].IdTextureDetail = -1;
			no ++;

			Face [no].a.i = i;
			Face [no].a.j = j;

			Face [no].b.i = i+1;
			Face [no].b.j = j+1;

			Face [no].c.i = i+1;
			Face [no].c.j = j;
			Face [no].IdTextureDetail = -1;
			no ++;
		}
	}

	// calcul des normal des facettes
	NormalFacette ();
}



// Effectue une RAZ des IdTexture de facette dont la valeur est egale  IdTexture
void CMakeTerrainDoc :: ResetFacetteTexture (int IdTexture)
{
	int i,j,no;
	no = 0;
	for (j = 0; j < NbPoint-1; j++)
	{
		for (i = 0; i < NbPoint-1; i++)
		{
			// Deuxieme triangle
			if (Face [no].IdTextureDetail == IdTexture) Face [no].IdTextureDetail = -1;
			no ++;
			if (Face [no].IdTextureDetail == IdTexture) Face [no].IdTextureDetail = -1;
			no ++;
		}
	}
}





// Fixe le nombre de point
BOOL CMakeTerrainDoc :: SetTotalPoint (int val)
{
	if (val < 2 || val > NPOINT) return FALSE;
	NbPoint = val;
	Reset (FALSE);
	return TRUE;
}
	
// Fixe une nouvelle largeur de cote
void CMakeTerrainDoc :: SetLargeur (int val)
{
	if (val < 1) return;
	Largeur = val;
	Reset (FALSE);
}

POINTASC_TERRE * CMakeTerrainDoc :: GetPoint (int i,int j)
{
	if (i < 0 || i >= NbPoint || j < 0 || j >= NbPoint) return NULL;
	return &Tab [i] [j];
}

// Initialise la heuteur d'un point
void CMakeTerrainDoc :: SetPointY (int i,int j,int Hauteur)
{
	if (i < 0 || i >= NbPoint || j < 0 || j >= NbPoint) return;
	POINTASC_TERRE * pt;
	pt = GetPoint (i,j);
	pt->y = Hauteur;
//	HMinMax (Hauteur);
	SetPointColor (i,j,Hauteur);

	ModifPoint (i,j);

}

void CMakeTerrainDoc :: SetMultiPointY (int indx,int indy,int np,int Hauteur)
{
	POINTASC_TERRE * pt;
	int i,j;
	int indxbis;
	int indybis;

	if (indx < 0 || indx >= NbPoint || indy < 0 || indy >= NbPoint) return;

	indxbis = indx + np;
	indybis = indy + np;

	if (indxbis >= NbPoint) indxbis = NbPoint-1;
	if (indybis >= NbPoint) indybis = NbPoint-1;

	for (i = indx; i <= indxbis; i++)
	for (j = indy; j <= indybis; j++)
	{
		pt = GetPoint (i,j);
		
	if (!pt)
	{
		CString str;
		str.Format ("i:%d  j:%d",i,j);
		AfxMessageBox (str);
		return;
	}
		
		pt->y = Hauteur;
//		HMinMax (Hauteur);

		SetPointColor (i,j,Hauteur);

	}

	
		for (i = indx; i <= indxbis; i++)
		for (j = indy; j <= indybis; j++)
			ModifPoint (i,j);
}

/*
void CMakeTerrainDoc :: HMinMax (int val)
{
	int Hmem;
	Hmem = Hmax;
	if (val < HauteurMin) HauteurMin = val;
	else if (val > HauteurMax) HauteurMax = val;
	Hmax = HauteurMax - HauteurMin;
	if (Hmem != Hmax) Reset (FALSE);

}
*/

void CMakeTerrainDoc :: SetPointColor (int i,int j,float val)
{
	float hauteurSet;
	BYTE r,g,b;
	
	hauteurSet = (val / Hauteur) * 255;
	if (hauteurSet > 255) hauteurSet = 255;
	else if (hauteurSet < 0) hauteurSet = 0;
	r = (BYTE) hauteurSet;
	g = (BYTE) hauteurSet;
	b = (BYTE) hauteurSet;


	pTexture->SetRgb (i,j,r,g,b);
}

void CMakeTerrainDoc :: UpdatePointColor ()
{
	int i,j;
	float hauteurSet;
	POINTASC_TERRE *pt;
	BYTE r,g,b;
	for (j = 0; j < NbPoint-1; j++)
	{
		for (i = 0; i < NbPoint-1; i++)
		{
			pt = GetPoint (i,j);
			hauteurSet = (pt->y / Hauteur) * 255;
			if (hauteurSet > 255) hauteurSet = 255;
			else if (hauteurSet < 0) hauteurSet = 0;
			pTexture->SetRgb (i,j,(BYTE) hauteurSet,(BYTE) hauteurSet,(BYTE) hauteurSet);
		}
	}

	// Mise  jour du bitmap de dessin 2D
	UpdateBitmap2D ();

}




// Calcul les normal de chaque FACETTE_TERRE
void CMakeTerrainDoc :: NormalFacette ()
{
	int no;
	for (no = 0; no < NbFacette; no++)
		NormalFacette (no);
}


// Calcul la normal d'une FACETTE_TERRE
void CMakeTerrainDoc :: NormalFacette (int no)
{
	POINTASC_TERRE * pt1;
	POINTASC_TERRE * pt2;
	POINTASC_TERRE * pt3;

	POINTASC_TERRE Vec1;
	POINTASC_TERRE Vec2;

	if (no < 0) return;

	// Prendre les points
	pt1 = GetPoint (Face [no].a.i,Face [no].a.j);
	pt2 = GetPoint (Face [no].b.i,Face [no].b.j);
	pt3 = GetPoint (Face [no].c.i,Face [no].c.j);


	// Calcul du vecteur N1
	Vec1.x = pt2->x - pt1->x;
	Vec1.y = pt2->y - pt1->y;
	Vec1.z = pt2->z - pt1->z;

	// Calcul du vecteur N2
	Vec2.x = pt3->x - pt1->x;
	Vec2.y = pt3->y - pt1->y;
	Vec2.z = pt3->z - pt1->z;



	// Calcul du vecteur normal
	POINTASC ptNorm;
	ptNorm = Normal (Vec1.x,Vec1.y,Vec1.z,Vec2.x,Vec2.y,Vec2.z);
	Face [no].n.x = ptNorm.x;
	Face [no].n.y = ptNorm.y;
	Face [no].n.z = ptNorm.z;


	Normalise (Face [no].n.x,Face [no].n.y,Face [no].n.z);
	
}


// Permet de normaliser un vecteur
void CMakeTerrainDoc::Normalise (GLfloat &x,GLfloat &y,GLfloat &z)
{
	GLfloat len;
	len = sqrt (x*x + y*y + z*z);
	if (len == 0) len = 1;
	x /= len;
	y /= len;
	z /= len;
}


BOOL CMakeTerrainDoc::OnSaveDocument(LPCTSTR lpszPathName) 
{
	// TODO: Add your specialized code here and/or call the base class
	return Save (lpszPathName);
}


BOOL CMakeTerrainDoc::Save (CString FileName)
{
	return TRUE;
}


// Modifie un point avec les triangles et les points qui l'entoure
void CMakeTerrainDoc::ModifPoint (int i,int j)
{
	POINTASC_TERRE * pt;
	int iFacette;
	int k;
	
	// Prendre le point 
	pt = GetPoint (i,j);

	
	// Calcul des vecteurs normaux des facettes autour du point
	for (k = 0; k < 6; k++)  
	{
		iFacette = GetAroundFacette(k,i,j);
		if (iFacette != -1) NormalFacette (iFacette);
	}

	if (bRender)
	{
		// Calcul des vecteurs normaux des points qui lie les facettes
		// autour du point i,j. la normal du point i,j va etre recalcul en mme temps.
		for (k = 0; k < 6; k++) 
		{
			iFacette = GetAroundFacette(k,i,j);
			if (iFacette != -1)
			{
				// Calcule des normal de point des facettes
				NormalPoint (Face [iFacette].a.i,Face [iFacette].a.j);
				NormalPoint (Face [iFacette].b.i,Face [iFacette].b.j);
				NormalPoint (Face [iFacette].c.i,Face [iFacette].c.j);
			}
		}
	}
}









// Retourne une struct de FACETTE_TERRE
FACETTE_TERRE * CMakeTerrainDoc :: GetFacette (int i)
{
	if (i < 0 || i >= NbFacette) return NULL;
	return &Face [i];
}




// Modifie les attributs du terrain
void CMakeTerrainDoc :: ConfigTerrain ()
{
	CDlgConfigTerrain Dlg;

	Dlg.Hauteur = Hauteur;
	Dlg.Largeur = Largeur;
	if (Dlg.DoModal () == IDCANCEL) return;
	Hauteur = Dlg.Hauteur;
	if (Dlg.Largeur != Largeur) SetLargeur (Dlg.Largeur);
}

// Modifie les attributs du terrain
void CMakeTerrainDoc :: Render ()
{
	int i,j,k,iFacette;
	
	// Prendre le point 
	for (i = 0; i < NbPoint ; i++)
	for (j = 0; j < NbPoint ; j++)
	{

		// Calcul des vecteurs normaux des points qui lie les facettes
		// autour du point i,j. la normal du point i,j va etre recalcul en mme temps.
		for (k = 0; k < 6; k++) 
		{

			iFacette = GetAroundFacette(k,i,j);
			if (iFacette != -1)
			{
				// Calcule des normal de point des facettes
				NormalPoint (Face [iFacette].a.i,Face [iFacette].a.j);
				NormalPoint (Face [iFacette].b.i,Face [iFacette].b.j);
				NormalPoint (Face [iFacette].c.i,Face [iFacette].c.j);
			}
		}
	}
}





// Cration d'une nouvelle liste de vertex pour un vaisseau
GLint CMakeTerrainDoc :: GenObjList(Objet * obj)
{
	FACETTE * pFacette;
	POINTASC *p1,*p2,*p3;
	int i,NbFacette;
	GLint lid;


	NbFacette = obj->GetNbFacette ();
	
	lid=glGenLists(1);
	glNewList(lid, GL_COMPILE);


	glEnable (GL_CULL_FACE);
	glColor4f (1,1,1,1);


	for (i = 0; i < NbFacette; i ++){

		pFacette = obj->GetFacette (i);

		p1 = obj->GetPoint (pFacette->a);
		p2 = obj->GetPoint (pFacette->b);
		p3 = obj->GetPoint (pFacette->c);


		// Selectionner la texture qui va bien
		obj->SelectTexture (pFacette->TextIndex);

		glBegin (GL_TRIANGLES);

		SetNormal (pFacette,1);
		SetTextureCoord (obj->GetTextCoord(pFacette->Tcoord [0]));
		glVertex3f(p1->x,p1->y,p1->z);

		SetNormal (pFacette,2);
		SetTextureCoord (obj->GetTextCoord(pFacette->Tcoord [1]));
		glVertex3f(p2->x,p2->y,p2->z);

		SetNormal (pFacette,3);
		SetTextureCoord (obj->GetTextCoord(pFacette->Tcoord [2]));
		glVertex3f(p3->x,p3->y,p3->z);

		glEnd();
	}


	glEndList();
	return lid;
}

void CMakeTerrainDoc :: SetTextureCoord (POINTASC * pt)
{
	if (pt) glTexCoord2f (pt->x,pt->y);
}


// Le numro indique la nature de la normal
// 0 normal de face
// 1 normal du point p1
// 2 normal du point p2
// 3 normal du point p3
void CMakeTerrainDoc ::  SetNormal (FACETTE * pFacette,int no)
{
	if (pFacette->norm == NULL) return;
	switch (no)
	{
	case 0:
	glNormal3f (pFacette->norm->nf.x,pFacette->norm->nf.y,pFacette->norm->nf.z);
		break;
	case 1:
	glNormal3f (pFacette->norm->p1.x,pFacette->norm->p1.y,pFacette->norm->p1.z);
		break;
	case 2:
	glNormal3f (pFacette->norm->p2.x,pFacette->norm->p2.y,pFacette->norm->p2.z);
		break;
	case 3:
	glNormal3f (pFacette->norm->p3.x,pFacette->norm->p3.y,pFacette->norm->p3.z);
		break;
	}

}







//SUBMATERIAL
Objet * CMakeTerrainDoc :: ChargerASEFile(CString FileName)
{
	int NbVertex,NbFaces,i;
	int a,b,c;
	FILE  *pFile;
	float x,y,z;
	CString str;
	Objet * obj;
	int nSubMat = 0;

	BOOL bFinded = FALSE;
	char * pt;
	CStringList VertexListe;
	CStringList FacetteListe;
	int count;
	int TextIndex;


	// --------------------------------------------------
	//			OUVERTURE DU FICHIER
	// --------------------------------------------------
	pFile = fopen (FileName,"r");
	
	if (!pFile)
	{
		AfxMessageBox ("Impossible d'ouvrir le fichier : "+FileName);
		return NULL;
	}

	// --------------------------------------------------
	//			RECHERCHE DU NOMBRE DE FACE ET DE POINT
	// --------------------------------------------------
	count = 0;
	bFinded = FALSE;
	while (!bFinded && fgets (ligne,sizeof (ligne),pFile))
	{
		if (strstr (ligne,NUMVERTEX))
		{
			pt = strstr (ligne,NUMVERTEX) + strlen (NUMVERTEX);
			if (atoi (pt) !=0) bFinded = TRUE;
		}
	}
	

	if (!bFinded) return NULL;
	
	// Extraction du nombre de Vertex
	pt = strstr (ligne,NUMVERTEX) + strlen (NUMVERTEX);
	NbVertex = atoi (pt);

	
	// Lire une ligne pour extraire le nombre de face
	fgets (ligne,sizeof (ligne),pFile);
	pt = strstr (ligne,NUMFACES) + strlen (NUMFACES);
	NbFaces = atoi (pt);


	str.Format ("%d Vertext et %d Facettes",NbVertex,NbFaces);
	VertexListe.AddTail (str);

	// --------------------------------------------------
	//			ALLOCATION DE L'OBJET
	// --------------------------------------------------
	obj = new Objet (NbVertex,NbFaces);
	obj->SetFileName (FileName);


	
	
	
	
	// --------------------------------------------------
	//			RECHERCHE DE MATERIEL
	// --------------------------------------------------
	// Repositionner au dbut du fichier
	fseek (pFile,0,SEEK_SET);
	bFinded = FALSE;
	while (!bFinded && fgets (ligne,sizeof (ligne),pFile))
	{
		if (strstr (ligne,MATERIALLIST)) bFinded = TRUE;
	}

	// Si on a trouv alors on extrait les noms de Textures
	if (bFinded)
	{

		// Recherche du nombre de sous materiel
		bFinded = FALSE;
		while (!bFinded && fgets (ligne,sizeof (ligne),pFile))
		{
			pt = strstr (ligne,NBSUBMATERIAL);
			if (pt) bFinded = TRUE;
		}

				
		if (bFinded)
		{
			nSubMat = atoi (pt+strlen (NBSUBMATERIAL));
			
			for (i = 0; i < nSubMat; i++)
			{
				fseek (pFile,0,SEEK_SET);
				bFinded = FALSE;
				str.Format ("%s%d",SUBMATERIAL,i);
				while (!bFinded && fgets (ligne,sizeof (ligne),pFile))
				{
					pt = strstr (ligne,str);
					if (pt) bFinded = TRUE;
				}

				// Si on a rien trouv alors on continue
				if (!bFinded) continue;


				// Recherche de marque BITMAP
				bFinded = FALSE;
				str.Format ("%s%d",SUBMATERIAL,i+1);
				while (!bFinded && fgets (ligne,sizeof (ligne),pFile))
				{
					// Recherhce de marque sous materiel superieur
					// Si on a trouv on 
					pt = strstr (ligne,str);
					// Si on a trouv un sous materiel superieur lors on passse
					if (pt) break;
					pt = strstr (ligne,TEXTUREFILENAME);
					// Si on a trouv alors on extrait les noms de Textures
					if (pt) 
					{
						str = pt+strlen (TEXTUREFILENAME);
						obj->AddTexture (str,i);
						bFinded = TRUE;
					}
				}
			}
		}
		else
		// Si on a pas trouv on prend le premier materiel
		{
			fseek (pFile,0,SEEK_SET);
			i = 0;
			while (fgets (ligne,sizeof (ligne),pFile))
			{
				pt = strstr (ligne,TEXTUREFILENAME);
				if (pt)
				{
					str = pt+strlen (TEXTUREFILENAME);
					obj->AddTexture (str,1);
					break;
				}
			}
		}
	}

	
	// Repositionner au dbut du fichier
	fseek (pFile,0,SEEK_SET);
	
	// Lire une ligne pour passer la ligne : *MESH_VERTEX_LIST {
	//MESH_VERTEX_LIST
	bFinded = FALSE;
	while (!bFinded && fgets (ligne,sizeof (ligne),pFile))
	{
		pt = strstr (ligne,MESH_VERTEX_LIST);
		if (pt) bFinded = TRUE;
	}

	// Probleme alors on vire en detruisant tout
	if (!bFinded)
	{
		AfxMessageBox ("MESH_VERTEX_LIST pas trouv apres les materiels");
		delete obj;
		return NULL;
	}	

	
	// --------------------------------------------------
	//			LECTURE DES POINTS
	// --------------------------------------------------
	// *MESH_VERTEX    0	-25.5561	-37.1951	0.0000
	for (i = 0; i < NbVertex; i++)
	{
		sprintf (chaine,"Vertex %d:",i);
		bFinded = FALSE;
		
		// Lire la ligne n i pour le vertex n i
		fgets (ligne,sizeof (ligne),pFile);
		toFloat (ligne);

		// Extraction du point X
		pt = ligne + 21;
		sscanf(pt,"%f",&x);

		while (*pt != '\t') pt ++;
		pt += 1;
		sscanf(pt,"%f",&y);
		
		while (*pt != '\t') pt ++;
		pt += 1;
		sscanf(pt,"%f",&z);

		obj->SetPoint (i,x,y,z);

	}


	// Lire une ligne pour passer la ligne : *MESH_VERTEX_LIST {
	fgets (ligne,sizeof (ligne),pFile);
	fgets (ligne,sizeof (ligne),pFile);
	
	// --------------------------------------------------
	//			LECTURE DES FACES
	// --------------------------------------------------
	// Lecture des facettes
	// Face 39:    A:12 B:29 C:13 AB:0 BC:1 CA:1
	int npassage = 0;
	int indexmaxmem = -1;
	for (i = 0; i < NbFaces; i++)
	{

		
		a = 0;
		b = 0;
		c = 0;
		fgets (ligne,sizeof (ligne),pFile);
		
		// Extraction du point X
		pt = strstr (ligne,"A:") + 2;
		if (pt) a = atoi (pt);

		// Extraction du point Y
		pt = strstr (ligne,"B:") + 2;
		if (pt) b = atoi (pt);
		
		// Extraction du point Z
		pt = strstr (ligne,"C:") + 2;
		if (pt) c = atoi (pt);

		// Extraction du n de materiel
		pt = strstr (ligne,MESHMTLID) + strlen (MESHMTLID) + 1;
//		TextIndex = -1;
		TextIndex = 0;
		if (pt) TextIndex = atoi (pt);

		// Index de texture
		obj->SetFacette (i,a,b,c,TextIndex);

		// Memorisation de l'index max
		if (indexmaxmem < TextIndex)
		{
			npassage ++;
			indexmaxmem = TextIndex;
		}

	}


	// --------------------------------------------------
	//			INDEXE DE TEXTURE - FILE ET LISTE
	// --------------------------------------------------
	obj->MakeIndexeTexture ();




	// --------------------------------------------------
	//			COORDONNEE DE TEXTURE
	// --------------------------------------------------
	bFinded = FALSE;
	while (!bFinded && fgets (ligne,sizeof (ligne),pFile))
	{
		pt = strstr (ligne,MESHNUMTVERTEX);
		if (pt)
		{
			pt += strlen (MESHNUMTVERTEX);
			count = atoi (pt);
			bFinded = TRUE;
		}
	}

	// Si on a trouv on extrait les donnes
	if (bFinded)
	{

		// Allouer la memoire necessaire pour
		// les coordonnes des textures
		obj->AllocTextureCoord (count);

		// Chargement des coordonnes
		fgets (ligne,sizeof (ligne),pFile);	// passer une ligne *MESH_TVERTLIST 

		for (i = 0; i < count; i++)
		{
		
			// Lire la ligne n i pour le vertex n i
			fgets (ligne,sizeof (ligne),pFile);
//			toFloat (ligne);

			sprintf (chaine,"%s%d",MESHTVERT,i);
			
			pt = strstr (ligne,chaine);
			pt += strlen (chaine) + 1;

			// Extraction du point X
			sscanf(pt,"%f",&x);

			while (*pt != '\t') pt ++;
			pt += 1;
			sscanf(pt,"%f",&y);
		
			while (*pt != '\t') pt ++;
			pt += 1;
			sscanf(pt,"%f",&z);

			// Initialiser la donnes
			obj->SetTextCoord (i,x,y,z);
	
		}

		// --------------------------------------------------
		//			INDEXE FACETTE DE TEXTURE
		// --------------------------------------------------
		bFinded = FALSE;
		while (!bFinded && fgets (ligne,sizeof (ligne),pFile))
		{
			pt = strstr (ligne,MESH_NUMTVFACES);
			if (pt)
			{
				pt += strlen (MESH_NUMTVFACES);
				count = atoi (pt);
				bFinded = TRUE;
			}
		}

		// Si on a trouv alors on charge les indexes
		// De texture de coordonnes
		if (bFinded)
		{
			// Pour passer le marqueur de liste
			fgets (ligne,sizeof (ligne),pFile);
			for (i = 0; i < count; i ++)
			{
				// Lire la ligne n i pour le vertex n i
				fgets (ligne,sizeof (ligne),pFile);

				sprintf (chaine,"%s%d",MESH_TFACE,i);
				pt = strstr (ligne,chaine);
				pt += strlen (chaine) + 1;

				// Extraction de la coordonne du point A
				a = atoi (pt);
				while (* pt != '\t')	pt++;
				pt += 1;

				// Extraction de la coordonne du point B
				b = atoi (pt);
				while (* pt != '\t')	pt++;
				pt += 1;
		
				// Extraction de la coordonne du point c
				c = atoi (pt);

				// Initialiser la donnes
				obj->SetTextCoordIndex (i,a,b,c);
			}
		}
	}



	// --------------------------------------------------
	//			CHARGEMENT DES NORMALES
	// --------------------------------------------------
	bFinded = FALSE;
	while (!bFinded && fgets (ligne,sizeof (ligne),pFile))
	{
		if (strstr (ligne,MESHNORMALS)) bFinded = TRUE;
	}

	
	// Si on a trouv alors on charge les coordone de texture
	// pour chaque point
	if (bFinded)
		for (i = 0; i < count; i++) obj->SetFacetteNormale (i,ReadNormal (i,pFile));

	
	// --------------------------------------------------
	//			FERMETURE DU FICHIER
	// --------------------------------------------------
	fclose (pFile);

	// Initialiser les point de collision
	obj->InitCubeCollision ();

	// Gnrer la liste d'instruction compile
	obj->noCallListe = GenObjList (obj);

  return obj;
}


NORMAL * CMakeTerrainDoc :: ReadNormal (int i,FILE * pFile)
{
	float x,y,z;
	char * pt;
	NORMAL * norm = NULL;
	HGLOBAL hglb;
	hglb = GlobalAlloc (GPTR,sizeof (NORMAL));
	norm = (NORMAL *) GlobalLock (hglb);

	if (norm == NULL) return NULL;

	

	// Lire la ligne n i pour le vertex n i
	fgets (ligne,sizeof (ligne),pFile);
//	toFloat (ligne);
	sprintf (chaine,"%s%d",MESHFACENORMAL,i);
	pt = strstr (ligne,chaine);
	
	
	if (!pt) return NULL;
	
	
	pt += strlen (chaine) + 1;
	
	// FACETTE
	// Extraction du point X
	sscanf(pt,"%f",&x);
	while (*pt != '\t') pt ++;
	pt += 1;
	sscanf(pt,"%f",&y);
	while (*pt != '\t') pt ++;
	pt += 1;
	sscanf(pt,"%f",&z);

	norm->nf.x = x;
	norm->nf.y = y;
	norm->nf.z = z;

	// P1
	fgets (ligne,sizeof (ligne),pFile);
//	toFloat (ligne);
	sprintf (chaine,"%s ",MESH_VERTEXNORMAL,i);
	pt = strstr (ligne,chaine) + strlen (chaine);
	while (*pt != '\t') pt ++; pt +=1;
	sscanf(pt,"%f",&x);
	
	while (*pt != '\t') pt ++; pt +=1;
	sscanf(pt,"%f",&y);

	while (*pt != '\t') pt ++; pt +=1;
	sscanf(pt,"%f",&z);

	norm->p1.x = x;
	norm->p1.y = y;
	norm->p1.z = z;

	// P2
	fgets (ligne,sizeof (ligne),pFile);
//	toFloat (ligne);
	sprintf (chaine,"%s ",MESH_VERTEXNORMAL,i);
	pt = strstr (ligne,chaine) + strlen (chaine);
	while (*pt != '\t') pt ++; pt +=1;
	sscanf(pt,"%f",&x);
	
	while (*pt != '\t') pt ++; pt +=1;
	sscanf(pt,"%f",&y);

	while (*pt != '\t') pt ++; pt +=1;
	sscanf(pt,"%f",&z);

	norm->p2.x = x;
	norm->p2.y = y;
	norm->p2.z = z;


	// P3
	fgets (ligne,sizeof (ligne),pFile);
//	toFloat (ligne);
	sprintf (chaine,"%s ",MESH_VERTEXNORMAL,i);
	pt = strstr (ligne,chaine) + strlen (chaine);
	while (*pt != '\t') pt ++; pt +=1;
	sscanf(pt,"%f",&x);
	
	while (*pt != '\t') pt ++; pt +=1;
	sscanf(pt,"%f",&y);

	while (*pt != '\t') pt ++; pt +=1;
	sscanf(pt,"%f",&z);
	
	norm->p3.x = x;
	norm->p3.y = y;
	norm->p3.z = z;

	return norm;
}


void CMakeTerrainDoc :: toFloat (char * ligne) {
	while (*ligne){
		if (*ligne == '.') *ligne = ',';
		ligne ++;
	}
}


// Deforme le terrain sur la hatueur des points
// par une valeur alatoir contenu entre [-Alea et Alea]
void CMakeTerrainDoc :: SetAleaHauteur (int Alea)
{
	int val;
	int i,j;
	POINTASC_TERRE * pt;
	for (i = 0; i < NbPoint; i ++)
	{
		for (j = 0; j < NbPoint; j ++)
		{
			pt = GetPoint (i,j);
			if (Alea) val = rand()%Alea;
			else val = 0;
			if (rand ()%10 < 5) val = -val;
			pt->y += val;
			SetPointColor (i,j,pt->y);
		}
	}
	NormalFacette ();
}

// Initialise l'identifiant de la texture  la case calcule par i et j
void CMakeTerrainDoc :: SetIdTexture (int ideb,int jdeb,int nFace,int IdTexture,int IndexTexture,BOOL bBlend)
{
	POINTASC_TERRE *p1,*p2,*p3;
	int Case;	
	int iFacette;
	int i,j;
	int ifin,jfin;
	BOOL bAffecter;

	if (ideb < 0) ideb = 0;
	if (jdeb < 0) jdeb = 0;
	if (ideb >= NbPoint-1)	ideb = NbPoint - 2;
	if (jdeb >= NbPoint-1)	jdeb = NbPoint - 2;
	if (nFace == 1) nFace = 0;


	ifin = ideb + nFace;
	jfin = jdeb + nFace;

	if (ifin < 0) ifin = 0;
	if (jfin < 0) jfin = 0;
	if (ifin >= NbPoint-1)	ifin = NbPoint - 2;
	if (jfin >= NbPoint-1)	jfin = NbPoint - 2;




	for (i = ideb; i <= ifin; i++)
	{
		for (j = jdeb; j <= jfin; j++)
		{
			Case = (j * (NbPoint - 1)) + i;
			iFacette = (Case * 2)+1;

			
			p1 = GetPoint (Face [iFacette].a.i,Face [iFacette].a.j);
			p2 = GetPoint (Face [iFacette].b.i,Face [iFacette].b.j);
			p3 = GetPoint (Face [iFacette].c.i,Face [iFacette].c.j);
			
			
			switch (TypePlaquage)
			{
			case TEXTURE_NIV_EGALE:bAffecter = CheckPointInTriangle (p1->y,p2->y,p3->y);break;
			default : 
				bAffecter = TRUE;
				if (!CheckPointByNiveau(p1)||!CheckPointByNiveau(p2)||!CheckPointByNiveau(p3)) bAffecter = FALSE;
				break;
			}

			if (bAffecter)
			{
				p1 = GetPoint (Face [iFacette-1].a.i,Face [iFacette-1].a.j);
				p2 = GetPoint (Face [iFacette-1].b.i,Face [iFacette-1].b.j);
				p3 = GetPoint (Face [iFacette-1].c.i,Face [iFacette-1].c.j);
				switch (TypePlaquage)
				{
				default : 
					bAffecter = TRUE;
					if (!CheckPointByNiveau(p1)||!CheckPointByNiveau(p2)||!CheckPointByNiveau(p3)) bAffecter = FALSE;
					break;
				}
			}

	
			if (bAffecter)
			{
				if (iFacette%2)
				{

					if (bBlend == FALSE)
					{
						Face [iFacette].IdTextureDetail = IdTexture;
						Face [iFacette-1].IdTextureDetail = IdTexture;
						Face [iFacette].IndexTextureDetail = IndexTexture;
						Face [iFacette-1].IndexTextureDetail = IndexTexture;
					}
				}
			}

		}
	}
}



void CMakeTerrainDoc :: SetIdTexture (int icase,int jcase,int IdTexture,int IndexTexture)
{
	POINTASC_TERRE *p1,*p2,*p3;
	int Case;	
	int iFacette;
	BOOL bAffecter;

	Case = (jcase * (NbPoint - 1)) + icase;
	iFacette = (Case * 2)+1;

			
	p1 = GetPoint (Face [iFacette].a.i,Face [iFacette].a.j);
	p2 = GetPoint (Face [iFacette].b.i,Face [iFacette].b.j);
	p3 = GetPoint (Face [iFacette].c.i,Face [iFacette].c.j);
			
			
	switch (TypePlaquage)
	{
		case TEXTURE_NIV_EGALE:bAffecter = CheckPointInTriangle (p1->y,p2->y,p3->y);break;
		default : 
			bAffecter = TRUE;
			if (!CheckPointByNiveau(p1)||!CheckPointByNiveau(p2)||!CheckPointByNiveau(p3)) bAffecter = FALSE;
			break;
	}

	if (bAffecter)
	{
		p1 = GetPoint (Face [iFacette-1].a.i,Face [iFacette-1].a.j);
		p2 = GetPoint (Face [iFacette-1].b.i,Face [iFacette-1].b.j);
		p3 = GetPoint (Face [iFacette-1].c.i,Face [iFacette-1].c.j);
		switch (TypePlaquage)
		{
		default : 
			bAffecter = TRUE;
			if (!CheckPointByNiveau(p1)||!CheckPointByNiveau(p2)||!CheckPointByNiveau(p3)) bAffecter = FALSE;
			break;
		}
	}

	
	if (bAffecter)
	{
		if (iFacette%2)
		{
			Face [iFacette].IdTextureDetail = IdTexture;
			Face [iFacette-1].IdTextureDetail = IdTexture;
			Face [iFacette].IndexTextureDetail = IndexTexture;
			Face [iFacette-1].IndexTextureDetail = IndexTexture;
		}
	}
}



// Voir si une hauteur se tsitue dans un triangle
BOOL CMakeTerrainDoc :: CheckPointInTriangle (float y1,float y2,float y3)
{
	float yTop,yBottom;

	yTop = y1;
	if (yTop < y2) yTop = y2;
	if (yTop < y3) yTop = y3;

	yBottom = y1;
	if (yBottom > y2) yBottom = y2;
	if (yBottom > y3) yBottom = y3;

	if (Hauteur > yBottom-1 && Hauteur < yTop+1) return TRUE;
	return FALSE;
}

// Cette mthode permet de retrouver la facette  partir d'un point
FACETTE_TERRE * CMakeTerrainDoc :: GetFacette (float x,float z)
{
	int iFacette;
	int Case;
	int i,j;
	int NbCase;
	float dist1,dist2;
	NbCase = NbPoint-1;

	i = (x - Dep) / Largeur;
	j = (z - Dep) / Largeur;

	if (i < 0) i = 0;
	if (j < 0) j = 0;
	if (i >= NbPoint-2) i = NbPoint-2;
	if (j >= NbPoint-2) j = NbPoint-2;

	POINTASC_TERRE * pttri;
	pttri = GetPoint (i+1,j);

	dist1 = sqrt ((x-pttri->x)*(x-pttri->x) + (z-pttri->z)*(z-pttri->z));

	pttri = GetPoint (i,j+1);
	dist2 = sqrt ((x-pttri->x)*(x-pttri->x) + (z-pttri->z)*(z-pttri->z));

	Case = (j * NbCase) + i;
	iFacette = (Case * 2)+1;

	if (dist1 > dist2)	iFacette ++;

	return  &Face [iFacette];

}



// Regarde si un poit est < ou > ou = ou partout
// pour l'application des textures.
BOOL CMakeTerrainDoc :: CheckPointByNiveau (int i,int j)
{
	POINTASC_TERRE * pt;
	pt = GetPoint (i,j);
	return CheckPointByNiveau (pt);
}
	

BOOL CMakeTerrainDoc :: CheckPointByNiveau (POINTASC_TERRE * pt)
{
	if (!pt) return FALSE;
	BOOL bRet;
	bRet = TRUE;
	switch (TypePlaquage)
	{
		case TEXTURE_NIV_PARTOUT:break;
		case TEXTURE_NIV_EGALE:bRet = (pt->y == Hauteur) ? TRUE : FALSE;break;
		case TEXTURE_NIV_DESSOUS:bRet = (pt->y < Hauteur) ? TRUE : FALSE;break;
		case TEXTURE_NIV_DESSUS:bRet = (pt->y > Hauteur) ? TRUE : FALSE;break;
	}
	return bRet;
}

float CMakeTerrainDoc :: Moyenne (POINTASC_TERRE * pt1,POINTASC_TERRE * pt2,POINTASC_TERRE * pt3,POINTASC_TERRE * pt4)
{
	int nMoyenne = 1;
	float y = 0;
	if (pt1) 
	{
		nMoyenne ++;
		y += pt1->y;
	}

	if (pt2) 
	{
		nMoyenne ++;
		y += pt2->y;
	}

	if (pt3) 
	{
		nMoyenne ++;
		y += pt3->y;
	}

	if (pt4) 
	{
		nMoyenne ++;
		y += pt4->y;
	}

	y /= nMoyenne;
	return y;

}



void CMakeTerrainDoc :: Lineariser (void) {
	int x, z;
	int i, j;
	int bsize, csize;
	int rnd;
	POINTASC_TERRE * pt1;
	POINTASC_TERRE * pt2;
	POINTASC_TERRE * pt3;
	POINTASC_TERRE * pt4;
	POINTASC_TERRE * pt;
	
	
	bsize = NbPoint;
	csize = NbPoint/2;
	
	rnd = Hauteur;
	
	// Remove any roughness from the heightfield.
	float yf = 0;
	for(j=0; j<8; j++) {
		for(x=0; x<NbPoint; x++) {
			for(z=0; z<NbPoint; z++) {
				yf = 0;
				pt1 = GetPoint(x-1, z-1);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x, z-1);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x+1, z-1);
				if (pt1) yf += pt1->y;

				pt1 = GetPoint(x-1, z);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x, z);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x+1, z);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x-1, z+1);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x, z+1);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x+1, z+1);
				if (pt1) yf += pt1->y;

				pt = GetPoint (x,z);
				pt->y = yf / 9;

				SetPointColor (x,z,pt->y);
			}
		}
	} 

	// calcul des normal des facettes
	NormalFacette ();

	// Effectuer un nouveau render
	Render ();

	// Initialiser les bit du Bitmap
	UpdateBitmap2D ();

}


void CMakeTerrainDoc :: Random (void) {
	int x, z;
	int i, j;
	int bsize, csize;
	int rnd;
	POINTASC_TERRE * pt1;
	POINTASC_TERRE * pt2;
	POINTASC_TERRE * pt3;
	POINTASC_TERRE * pt4;
	POINTASC_TERRE * pt;
	
	// Reset du terrain
	Reset (TRUE);
	
	bsize = NbPoint;
	csize = NbPoint/2;
	
	rnd = Hauteur;
	
	while(csize > 0) {
		for(x=0; x<NbPoint; x+=bsize) {
			for(z=0; z<NbPoint; z+=bsize) {
				pt1 = GetPoint (x,z);
				pt2 = GetPoint (x+bsize,z);
				pt3 = GetPoint (x+bsize,z+bsize);
				pt4 = GetPoint (x,z+bsize);
				pt = GetPoint (x+csize,z+csize);
				pt->y = Moyenne (pt1,pt2,pt3,pt4) + rand()%(rnd+1) - (rnd/2);
			}
		}
		
		for(x=0; x<NbPoint; x+=bsize) {
			for(z=0; z<NbPoint; z+=bsize) {
				pt1 = GetPoint (x,z);
				pt2 = GetPoint (x+bsize,z);
				pt3 = GetPoint (x+csize,z+csize);
				pt4 = GetPoint (x+csize,z-bsize);
				pt = GetPoint (x+csize,z);
				pt->y = Moyenne (pt1,pt2,pt3,pt4) + rand()%(rnd+1) - (rnd/2);

			}
		}
		
		for(x=0; x<NbPoint; x+=bsize) {
			for(z=0; z<NbPoint; z+=bsize) {
				pt1 = GetPoint (x,z);
				pt2 = GetPoint (x,z+bsize);
				pt3 = GetPoint (x-csize,z+csize);
				pt4 = GetPoint (x+csize,z+bsize);
				pt = GetPoint (x,z+csize);
				pt->y = Moyenne (pt1,pt2,pt3,pt4) + rand()%(rnd+1) - (rnd/2);

			}
		} 
		
		bsize /= 2;
		csize /= 2;
		if(csize < 16) rnd /= 2;
	}
	
	
	for(x=0; x<NbPoint; x++)
		for(z=0; z<NbPoint; z++) {
			pt = GetPoint (x,z);
			if (pt->y < 0) pt->y *= -1;
		}
	
	// Remove any roughness from the heightfield.
	float yf = 0;
	for(j=0; j<8; j++) {
		for(x=0; x<NbPoint; x++) {
			for(z=0; z<NbPoint; z++) {

				yf = 0;
				pt1 = GetPoint(x-1, z-1);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x, z-1);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x+1, z-1);
				if (pt1) yf += pt1->y;

				pt1 = GetPoint(x-1, z);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x, z);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x+1, z);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x-1, z+1);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x, z+1);
				if (pt1) yf += pt1->y;
				pt1 = GetPoint(x+1, z+1);
				if (pt1) yf += pt1->y;

				pt = GetPoint (x,z);
				pt->y = yf / 8;
				SetPointColor (x,z,pt->y);

				
			}
		}
	} 

	// calcul des normal des facettes
	NormalFacette ();

	// Effectuer un nouveau render
	if (bRender) Render ();

	// Initialiser les bit du Bitmap
	UpdateBitmap2D ();



}

// Mise  jour du bitmap pour la vue 2D
BOOL CMakeTerrainDoc::UpdateBitmap2D ()
{
	BOOL bRet;
	int size;
	if (pTexture == NULL) return FALSE;
	size = NbPoint*NbPoint*NBYTE;
	bRet = (BmpLand.SetBitmapBits (size,pTexture->GetBit ()) != 0) ? TRUE : FALSE;
	return bRet;
}

void CMakeTerrainDoc::ApplyTextureAll(int IdTexure,int IndexTexture) 
{
	int NbPoint; 
	int i,j;
	NbPoint = GetTotalPoint ();
	for (i = 0; i <= NbPoint - 2; i++)
	{
		for (j = 0; j <= NbPoint - 2; j++)
			SetIdTexture (i,j,IdTexure,IndexTexture);
	}
}



// Chargement d'un terrain par mapping
BOOL CMakeTerrainDoc :: LoadMapping (CString FileName)
{
	int Size;
	TextureInfo * pTextTmp;

	pTextTmp = new TextureInfo (FileName,TRUE);
	if (pTextTmp == NULL) return FALSE;

	if (!pTextTmp->IsLoading ())
	{
		delete pTextTmp;
		return FALSE;
	}

	// initialisation du terrain
	Size = pTextTmp->Width ();

	// Vrifier la taille de la texture.
	switch (Size)
	{
		case 32:case 64:case 128:
		case 256:case 512:case 1024:
		break;

		default:
		delete pTextTmp;
		return FALSE;
	}

	if (GetTotalPoint () != Size)
	{
		if (!NewTerrain (Size))
		{
			AfxMessageBox ("Impossible de crer le terrain");
			PostQuitMessage (0);
			return FALSE;
		}
	}


	pTexture->SetBit (pTextTmp->GetBit ());
	pTexture->GreyLevel ();
	delete pTextTmp;
	UpdateLandSacpeByMapping (pTexture);

	// calcul des normal des facettes
	NormalFacette ();

	// Effectuer un nouveau render
	if (bRender) Render ();

	// Mise  jour du bitmap
	UpdateBitmap2D ();

	return TRUE;
}


// Initialise les point du terrain par une texture
BOOL CMakeTerrainDoc :: UpdateLandSacpeByMapping (TextureInfo * pText)
{
	POINTASC_TERRE * pt;
	BYTE r,g,b;
	int i,j;
	float Coef,y,val;
	Coef = Hauteur / 255;
	for (i = 0; i < NbPoint; i++)
	for (j = 0; j < NbPoint; j++)
	{
		pText->GetRgb (i,j,r,g,b);
		val = (float) ((float) (r+b+g) / 3); 
		pt = GetPoint (i,j);
		pt->y = val * Coef;
	}
	
	return TRUE;
}


// Cette mthode permet d'initialiser les six indexs
// de facettes lies par un point
void CMakeTerrainDoc::NormalPoint  (int i,int j)
{
	int count,k,iFacette;
	POINTASC_TERRE * pt;

	pt = GetPoint (i,j);
	count = 0;
	
	// Normal  zero
	pt->n.x = 0;
	pt->n.y = 0;
	pt->n.z = 0;

	for (k = 0; k < 6; k++)
	{
		iFacette = GetAroundFacette(k,i,j);
		if (iFacette != -1)
		{
			count ++;
			pt->n.x += Face [iFacette].n.x;
			pt->n.y += Face [iFacette].n.y;
			pt->n.z += Face [iFacette].n.z;
		}

	}


	pt->n.x /= count;
	pt->n.y /= count;
	pt->n.z /= count;
	
	Normalise (pt->n.x,pt->n.y,pt->n.z);
}




// Cette mthode renvoit un index de facette autour d'un point.
// Si la valeur renvoy est -1 alors il ne faut rien faire avec cet index.
int CMakeTerrainDoc::GetAroundFacette(int index,int i,int j)
{
	int iFacette;
	int nbcase = NbPoint-1;
	
	// Initialisation des indexs
	iFacette = (((j * nbcase) + i) * 2)+1;
	
	switch (index)
	{
		case 0:
			if (i >= nbcase || j >= nbcase) return -1;
			break;

		case 1:
			if (i >= nbcase || j >= nbcase) return -1;
			iFacette --;
			break;
		case 2:
			if (i <= 0 || j >= nbcase) return -1;
			iFacette -= 2;
			break;

		case 3:
			if (i >= nbcase || j <= 0) return -1;
			iFacette -= (nbcase*2) +1;
			break;
		case 4:
			if (i <= 0 || j <= 0) return -1;
			iFacette -= (nbcase*2) +2;
			break;
		case 5:
			if (i <= 0 || j <= 0) return -1;
			iFacette -= (nbcase*2) +3;
			break;
	}
	if (iFacette >= NbFacette || iFacette < 0 ) iFacette = -1;
	return iFacette;
}



// Ancien Code Sur la normalisation des points
// avec le calcul des effets de lumire
/*

  
// Version de la structure POINTASC_TERRE avec
// le code ci dessous
typedef struct
{
	// Le point
	GLfloat x;
	GLfloat y;
	GLfloat z;

	float tx,ty;

	// Normale du point
	NORMAL_TERRE n;

	// Les facettes lies par le points
//	int iFace [6];

} POINTASC_TERRE;

  
	// Calcule la normale d'un point
void CMakeTerrainDoc::NormalPoint (int i,int j)
{
	int k,iFacette;
	int count;
	POINTASC_TERRE * pt;

	pt = GetPoint (i,j);

	pt->n.x = 0;
	pt->n.y = 0;
	pt->n.z = 0;


	count = 0;
	for (k = 0; k < 6; k++)
	{
		iFacette = pt->iFace [k];
		if (iFacette == -1) continue;

		pt->n.x += Face [iFacette].n.x;
		pt->n.y += Face [iFacette].n.y;
		pt->n.z += Face [iFacette].n.z;
		count ++;

	}

	
	pt->n.x /= count;
	pt->n.y /= count;
	pt->n.z /= count;
	
	Normalise (pt->n.x,pt->n.y,pt->n.z);
}

// Cette mthode permet d'initialiser les six indexs
// de facettes lies par un point
void CMakeTerrainDoc::InitPointAround ()
{
	POINTASC_TERRE * pt;
	int nbcase;
	int nbcasemoinsun;
	int Case;
	int iFacette;
	int i,j,k;

	nbcase = NbPoint - 1;
	nbcasemoinsun = nbcase - 1;

	for (i = 0; i < NbPoint; i ++)
	{
		for (j = 0; j < NbPoint; j ++)
		{

			pt = GetPoint (i,j);
			
			
			// Initialisation par defaut des indexs de FACETTE_TERRE  -1
			for (k = 0; k < 6; k++) pt->iFace [k] = -1;
			
			
			// Initialisation des indexs
			Case = (j * nbcase) + i;
			iFacette = (Case * 2)+1;

			if (i < nbcase && j < nbcase)
			{
			pt->iFace [0] = iFacette ;
			pt->iFace [1] = iFacette -1;
			}

			if (i > 0 && j < nbcase) pt->iFace [2] = iFacette -2;
			if (i < nbcase && j > 0) pt->iFace [3] = iFacette - (nbcase*2) -1;

			if (i > 0 && j > 0)
			{
				pt->iFace [4] = iFacette-(nbcase*2)-2;
				pt->iFace [5] = iFacette-(nbcase*2)-3;
			}

		}
	}

	for (j = 0; j < NbPoint; j ++)
	{
		for (i = 0; i < NbPoint; i ++)
		{
			pt = GetPoint (i,j);
			for (k = 0; k < 6; k++)
			{
				if (pt->iFace [k] >= NbFacette) pt->iFace [k] = -1;
				if (pt->iFace [k] < 0 ) pt->iFace [k] = -1;
			}

		}
	}
}

// Modifie un point avec les triangles et les points qui l'entoure
void CMakeTerrainDoc::ModifPoint (int i,int j)
{
	POINTASC_TERRE * pt;
	int iFacette;
	int k;
	
	// Prendre le point 
	pt = GetPoint (i,j);

	
	// Calcul des vecteurs normaux des facettes autour du point
	for (k = 0; k < 6; k++)  
		NormalFacette (pt->iFace [k]);

	if (bRender)
	{
		// Calcul des vecteurs normaux des points qui lie les facettes
		// autour du point i,j. la normal du point i,j va etre recalcul en mme temps.
		for (k = 0; k < 6; k++) 
		{
			// Ne rien faire si l'indice de FACETTE_TERRE est -1
			if (pt->iFace [k] == -1) continue;
		
			// Prendre les indices des points des facettes et calculer le vecteur normal
			iFacette = pt->iFace [k];
			if (iFacette  >= NbFacette) continue;


			// Calcule des normal de point des facettes
			NormalPoint (Face [iFacette].a.i,Face [iFacette].a.j);
			NormalPoint (Face [iFacette].b.i,Face [iFacette].b.j);
			NormalPoint (Face [iFacette].c.i,Face [iFacette].c.j);
		
		}
	}
}

// Modifie les attributs du terrain
void CMakeTerrainDoc :: Render ()
{
	int i,j,k;
	POINTASC_TERRE * pt;
	int iFacette;
	
	// Prendre le point 
	for (i = 0; i < NbPoint ; i++)
	for (j = 0; j < NbPoint ; j++)
	{
		pt = GetPoint (i,j);

		// Calcul des vecteurs normaux des points qui lie les facettes
		// autour du point i,j. la normal du point i,j va etre recalcul en mme temps.
		for (k = 0; k < 6; k++) 
		{
			// Ne rien faire si l'indice de FACETTE_TERRE est -1
			if (pt->iFace [k] == -1) continue;
		
			// Prendre les indices des points des facettes et calculer le vecteur normal
			iFacette = pt->iFace [k];
			if (iFacette  >= NbFacette) continue;


			// Calcule des normal de point des facettes
			NormalPoint (Face [iFacette].a.i,Face [iFacette].a.j);
			NormalPoint (Face [iFacette].b.i,Face [iFacette].b.j);
			NormalPoint (Face [iFacette].c.i,Face [iFacette].c.j);
		
		}

	}

}

*/