#ifndef TERRAIN_TYPE_H
#define TERRAIN_TYPE_H

#include "gl\gl.h"
#include "gl\glu.h"

#define NPOINT					256
#define NBYTE					4
#define NBIT					(8*NBYTE)


#define NFACETTE	((NPOINT-1)*(NPOINT-1)*2)

typedef struct
{
	GLfloat x;
	GLfloat y;
	GLfloat z;

} NORMAL_TERRE;



typedef struct
{
	// Le point
	GLfloat x;
	GLfloat y;
	GLfloat z;

	float tx,ty;

	// Normale du point
	NORMAL_TERRE n;

} POINTASC_TERRE;





typedef struct
{
	GLfloat x;
	GLfloat y;
	GLfloat z;
	BOOL bEnTir;
} MISSILE;


typedef struct
{
	int IdListe;
	int IdTexture;
} EXPLODELISTE;


typedef struct
{
	GLfloat x;
	GLfloat y;
	GLfloat z;

	GLfloat r;
	GLfloat g;
	GLfloat b;


} POINTASC;

typedef struct
{
	GLfloat x;
	GLfloat y;
	GLfloat z;
} POINT3D;


typedef struct
{
	POINT3D nf;
	POINT3D p1;
	POINT3D p2;
	POINT3D p3;

} NORMAL;


typedef struct
{
	int a;
	int b;
	int c;
	int TextIndex;

	// Index des coordonne de texture par point
	int Tcoord [3];

	NORMAL * norm;


} FACETTE;






typedef struct
{
	int i;
	int j;
} PTAB_TERRE;


typedef struct
{
	PTAB_TERRE a;	
	PTAB_TERRE b;	
	PTAB_TERRE c;
	NORMAL_TERRE n;
//	int IdTexture;
//	int IndexTexture;	// Indexe de texture dans la liste
	int IdTextureDetail;
	int IndexTextureDetail;	// Indexe de texture de detail dans la liste
} FACETTE_TERRE;


typedef struct
{
	GLfloat Ambient[4];
	GLfloat Diffuse[4];
	GLfloat Specular[4];
	GLfloat Emission[4];
	GLfloat Shineness[1];
} MATERIEL;


// Faire bouger la vue 3D
#define MANIP_ROTATION			1
#define MANIP_ZOOM				2
#define MANIP_MOVE				3
#define MANIP_MOVE_LUMIERE_XZ	4
#define MANIP_MOVE_LUMIERE_Y	5
#define MANIP_LARGEUR			6
#define MANIP_SHIP				7


// Les diffrents type de dessin
#define DESSIN_PINCEAU					0
#define DESSIN_ARRONDISSEMENT			1
#define DESSIN_COLLINE					2
#define DESSIN_COLLINE_CREUX			3
#define DESSIN_TEXTURE					4
#define DESSIN_APPLATIR					5 
#define DESSIN_LARGEUR					6 



#define DRAW_TERRAIN_HAUTEUR			1
#define DRAW_TERRAIN_TEXTURE			2

#define PINCEAU_SQUARE					1
#define PINCEAU_CIRCLE					2


#define TERRAIN_AFFICHAGE_FIL_DE_FER			1
#define TERRAIN_AFFICHAGE_FACETTE				2
#define TERRAIN_AFFICHAGE_POINT					3

#define TEXTURE_NIV_PARTOUT						0
#define TEXTURE_NIV_EGALE						1
#define TEXTURE_NIV_DESSOUS						2
#define TEXTURE_NIV_DESSUS						3



// Dfinition de la structure de tous les messages rseaux utiliss
// par l'application
typedef struct 
{
	int Type;
	unsigned long IdPlayer;
} MESSAGE_TYPE;


typedef struct 
{
	MESSAGE_TYPE Type;
	unsigned long	IdPlayerToLock;	// Identifiant rseaux du joueur cible
} MESSAGE_LOCK;


typedef struct 
{
	MESSAGE_TYPE Type;
	unsigned long	IdPlayerCible;	// Identifiant rseaux du joueur cible
	int no;					// Quelle type de missil il reprsente
	GLfloat vitesse;		// Vitesse initiales pour le calcul de dx,dy et dz
	GLfloat matrice [16];	// Matrice d'orientation
} MESSAGE_MISSILE_TELEGUIDE;


typedef struct 
{
	MESSAGE_TYPE Type;
	int no;					// Quelle type de missil il reprsente
	GLfloat vitesse;		// Vitesse initiales pour le calcul de dx,dy et dz
	GLfloat matrice [16];	// Matrice d'orientation
} MESSAGE_MISSIL_DIRECT;



typedef struct 
{
	MESSAGE_TYPE Type;
	GLfloat vitesse;		// Information de vitesse
	GLfloat matrice [16];	// Matrice d'orientation
} MESSAGE_SHIP;

typedef struct 
{
	MESSAGE_TYPE Type;
	GLfloat x;
	GLfloat y;
	GLfloat z;
} MESSAGE_SHIP_POSITION;

typedef struct 
{
	MESSAGE_TYPE Type;
	GLfloat incax;
	GLfloat incay;
	GLfloat incaz;
} MESSAGE_SHIP_PIVOT;

typedef struct 
{
	MESSAGE_TYPE Type;
	GLfloat vitesse;
} MESSAGE_SHIP_VITESSE;

// Structure de message utilis pour les objets anims
// genre comme :
//	Missile
//	Vaisseaux
typedef struct 
{
	MESSAGE_TYPE Type;
	GLfloat x;
	GLfloat y;
	GLfloat z;
} MESSAGE_EXPLOSION;


// Structure de message utilis pour les objets anims
// genre comme :
//	Missile
//	Vaisseaux
typedef struct 
{
	MESSAGE_TYPE Type;
	int no;
	GLfloat x;
	GLfloat y;
	GLfloat z;
	GLfloat matrice [16];
} MESSAGE_PLAYER_INFO;


// Dfinition des types de message
#define		MSG_MISSILE			  -1	// Cration missile directionnel
#define		MSG_MISSILE_TELEGUIDE -2	// Cration missile tlguid
#define		MSG_SHIP			  -3	// Information sur la position d'un joueur
#define		MSG_EXPLOSE			  -4	// Coordone pour une explosion
#define		MSG_NEW_PLAYER		  -5	// Notification d'un nouveau player
#define		MSG_DESTROY_PLAYER	  -6	// Notification de la destrucion d'un player
#define		MSG_SHIP_POSITION	  -7	// Position dun vaisseau
#define		MSG_LOCK			  -8
#define		MSG_UNLOCK			  -9
#define		MSG_SHIP_PIVOT			-10
#define		MSG_SHIP_VITESSE		  -11
#define		MSG_SHIP_VITESSE_BY_INC		-12
#define		MSG_SHIP_DEPLACE			-13

// Lorsqu'un joueur recoit MSG_GESTION_DEB, son ordinateur peut effectuer
// une gestion de donnes (faire avanc son vaisseau et ses missiles)
#define		MSG_GESTION_FIN				-14

// Lorsque l'ordinateur crateur recoit MSG_GESTION_FIN alors
// il compte le nombre de message recu et comparre avec le nombre de joueur
// si le nombre est >= alors on effectue une mise  jour du compteur  0
// et on renvoit le message MSG_GESTION_DEB  tous les autres pour effectuer un nouveau dplacement
#define		MSG_GESTION_DEB				-15



#define MAX(a,b) ((a) > (b) ? (a) : (b))
#define MIN(a,b) ((a) < (b) ? (a) : (b))

// Dfinition des diffrents timer utiliss
#define ID_TIMER_SCENE				1
#define ID_TIMER_EXPLOSION			2
#define TIMER_EXPLOSION_ELLAPSE		20

// Lecture des fichiers ASE
#define NUMVERTEX			"*MESH_NUMVERTEX"
#define NUMFACES			"*MESH_NUMFACES"
#define SUBMATERIAL			"*SUBMATERIAL "
#define TEXTUREFILENAME		"*BITMAP "
#define NBSUBMATERIAL		"*NUMSUBMTLS "

#define MATERIALLIST		"*MATERIAL_LIST"
#define MATERIALNAME		"*MATERIAL_NAME"
#define SUBMATERIAL			"*SUBMATERIAL "
#define MESH_VERTEX_LIST	"*MESH_VERTEX_LIST"
#define MESH_VERTEXNORMAL	"*MESH_VERTEXNORMAL"

#define MESHSMOOTHING		"*MESH_SMOOTHING"
#define MESHMTLID			"*MESH_MTLID"

#define MESHNUMTVERTEX		"*MESH_NUMTVERTEX "
#define MESHTVERTLIST		"*MESH_TVERTLIST"
#define MESHTVERT			"*MESH_TVERT "

#define MESHNORMALS			"*MESH_NORMALS"
#define MESHFACENORMAL		"*MESH_FACENORMAL "
#define GEOMOBJECT			"*GEOMOBJECT"


#define MESH_NUMTVFACES			"*MESH_NUMTVFACES "
#define MESH_TFACE				"*MESH_TFACE "

#define TRIMESH		"Tri-mesh"
#define VERTICES	"Vertices: "
#define FACES		"Faces: "

#define PI 3.141592654


#define TIME_ELLAPSE		10					// mili seconde
#define NBETOILE			1000

// Definition sur les missils
#define NMISSILE					20
#define MISSILE_NB_INC_MAX			100
#define MISSILE_NB_AUTO_INC_MAX		1000			// 150
#define VITESSE_MISSILE_MAX			120
#define VITESSE_AUTO_MISSILE_MAX	20			// 50
#define AUTOMISSILE_INC_ANGLE		3			// 8

// Angle de rotation quand un vaisseau est lock sur un autre
#define LOCK_INC_ANGLE				5

// Pour les camra
#define	DISTANCE_CAMERA_MAX			20


#define INC_VITESSE					2
#define NPOINTCOLLISION				8
#define ZOOMCOEF					10
#define ANINC						0.5
#define ANINC_BY_KEY				5


#define AXELONGUEUR		150
#define AXEMONDE	1000


#define GAUCHE_HAUT_DEVANT		0
#define DROITE_HAUT_DEVANT		1
#define DROITE_HAUT_ARRIERE		3
#define GAUCHE_HAUT_ARRIERE		2

#define GAUCHE_BAS_DEVANT		4
#define DROITE_BAS_DEVANT		5
#define DROITE_BAS_ARRIERE		7
#define GAUCHE_BAS_ARRIERE		6

// Les diffrents type d'objet utilis
#define	ID_OBJET_MISSILE				0
#define	ID_OBJET_MISSILE_TELEGUIDE		1
#define	ID_OBJET_SHIP					2
#define	ID_OBJET_EXPLOSION				3
	
#define SELECT_NEXT						TRUE
#define SELECT_PREV						FALSE

#define	ID_TIMER_AUTO_AVANCE	100




#endif

